/***************************************************************************
 *   Copyright (C) 2005 by Angelo Rosiello   *
 *   angelo@rosiello.org   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "arcs.h"

arcs::arcs()
{
  byte_order_test[0]=1;
  byte_order_test[1]=0;
  byte_order_test[2]=0;
  byte_order_test[3]=0;
}


arcs::~arcs()
{
}

int arcs::encrypt( QString source_file, QString dest_file, QString pwd )
{
  return core( (char *)source_file.latin1(), (char *)dest_file.latin1(), (char *)pwd.latin1() );
}

int arcs::decrypt( QString source_file, QString dest_file, QString pwd )
{
  return core( (char *)source_file.latin1(), (char *)dest_file.latin1(), (char *)pwd.latin1() );
}

void arcs::MD5_Init( )
{
    MD5_A = 0x67452301;
    MD5_B = 0xefcdab89;
    MD5_C = 0x98badcfe;
    MD5_D = 0x10325476;
    MD5_Nl = 0;
    MD5_num = 0;
}

void arcs::md5_block( )
{
    unsigned32 A=MD5_A, B=MD5_B, C=MD5_C, D=MD5_D;
    int i;
/*
 * Here I re-write the buffer so that it now behaves as if it is
 * an array of 32-bit words in native computer representation. On
 * many machines the code here will have no effect at all apart from
 * consuming a little time. I do a little test first to see if
 * it is really needed.
 */
    unsigned32 *p = MD5_data;
    unsigned char *q = (unsigned char *)p;
    if (((unsigned32 *)byte_order_test)[0] != 1)
    {   for (i=0; i<MD5_LBLOCK; i++)
        {   unsigned32 w = *q++;
            w |= *q++ << 8;
            w |= *q++ << 16;
            w |= *q++ << 24;
            *p++ = w;
        }
    }
    p = MD5_data;
    /* Round 0 */
    R0(A,B,C,D,p[ 0], 7,0xd76aa478); R0(D,A,B,C,p[ 1],12,0xe8c7b756);
    R0(C,D,A,B,p[ 2],17,0x242070db); R0(B,C,D,A,p[ 3],22,0xc1bdceee);
    R0(A,B,C,D,p[ 4], 7,0xf57c0faf); R0(D,A,B,C,p[ 5],12,0x4787c62a);
    R0(C,D,A,B,p[ 6],17,0xa8304613); R0(B,C,D,A,p[ 7],22,0xfd469501);
    R0(A,B,C,D,p[ 8], 7,0x698098d8); R0(D,A,B,C,p[ 9],12,0x8b44f7af);
    R0(C,D,A,B,p[10],17,0xffff5bb1); R0(B,C,D,A,p[11],22,0x895cd7be);
    R0(A,B,C,D,p[12], 7,0x6b901122); R0(D,A,B,C,p[13],12,0xfd987193);
    R0(C,D,A,B,p[14],17,0xa679438e); R0(B,C,D,A,p[15],22,0x49b40821);
    /* Round 1 */
    R1(A,B,C,D,p[ 1], 5,0xf61e2562); R1(D,A,B,C,p[ 6], 9,0xc040b340);
    R1(C,D,A,B,p[11],14,0x265e5a51); R1(B,C,D,A,p[ 0],20,0xe9b6c7aa);
    R1(A,B,C,D,p[ 5], 5,0xd62f105d); R1(D,A,B,C,p[10], 9,0x02441453);
    R1(C,D,A,B,p[15],14,0xd8a1e681); R1(B,C,D,A,p[ 4],20,0xe7d3fbc8);
    R1(A,B,C,D,p[ 9], 5,0x21e1cde6); R1(D,A,B,C,p[14], 9,0xc33707d6);
    R1(C,D,A,B,p[ 3],14,0xf4d50d87); R1(B,C,D,A,p[ 8],20,0x455a14ed);
    R1(A,B,C,D,p[13], 5,0xa9e3e905); R1(D,A,B,C,p[ 2], 9,0xfcefa3f8);
    R1(C,D,A,B,p[ 7],14,0x676f02d9); R1(B,C,D,A,p[12],20,0x8d2a4c8a);
    /* Round 2 */
    R2(A,B,C,D,p[ 5], 4,0xfffa3942); R2(D,A,B,C,p[ 8],11,0x8771f681);
    R2(C,D,A,B,p[11],16,0x6d9d6122); R2(B,C,D,A,p[14],23,0xfde5380c);
    R2(A,B,C,D,p[ 1], 4,0xa4beea44); R2(D,A,B,C,p[ 4],11,0x4bdecfa9);
    R2(C,D,A,B,p[ 7],16,0xf6bb4b60); R2(B,C,D,A,p[10],23,0xbebfbc70);
    R2(A,B,C,D,p[13], 4,0x289b7ec6); R2(D,A,B,C,p[ 0],11,0xeaa127fa);
    R2(C,D,A,B,p[ 3],16,0xd4ef3085); R2(B,C,D,A,p[ 6],23,0x04881d05);
    R2(A,B,C,D,p[ 9], 4,0xd9d4d039); R2(D,A,B,C,p[12],11,0xe6db99e5);
    R2(C,D,A,B,p[15],16,0x1fa27cf8); R2(B,C,D,A,p[ 2],23,0xc4ac5665);
    /* Round 3 */
    R3(A,B,C,D,p[ 0], 6,0xf4292244); R3(D,A,B,C,p[ 7],10,0x432aff97);
    R3(C,D,A,B,p[14],15,0xab9423a7); R3(B,C,D,A,p[ 5],21,0xfc93a039);
    R3(A,B,C,D,p[12], 6,0x655b59c3); R3(D,A,B,C,p[ 3],10,0x8f0ccc92);
    R3(C,D,A,B,p[10],15,0xffeff47d); R3(B,C,D,A,p[ 1],21,0x85845dd1);
    R3(A,B,C,D,p[ 8], 6,0x6fa87e4f); R3(D,A,B,C,p[15],10,0xfe2ce6e0);
    R3(C,D,A,B,p[ 6],15,0xa3014314); R3(B,C,D,A,p[13],21,0x4e0811a1);
    R3(A,B,C,D,p[ 4], 6,0xf7537e82); R3(D,A,B,C,p[11],10,0xbd3af235);
    R3(C,D,A,B,p[ 2],15,0x2ad7d2bb); R3(B,C,D,A,p[ 9],21,0xeb86d391);

    MD5_A += A;
    MD5_B += B;
    MD5_C += C;
    MD5_D += D;
}

void arcs::MD5_Update(unsigned char *data, int len)
{
    unsigned char *p = (unsigned char *)MD5_data;

/*
 * The full MD5 procedure allows for encoding strings of up to
 * around 2^64 bits. I will restrict myself to 2^32 so I can just ignore
 * the high word of the bit-count.
 */
    MD5_Nl += len<<3;   /* Counts in BITS not BYTES here */

    while (len != 0)
    {   p[MD5_num++] = *data++;
        len--;
        if (MD5_num == MD5_CBLOCK)
        {   md5_block();
            MD5_num = 0;
        }
    }
}


void arcs::MD5_Final(unsigned char *md)
{
    unsigned32 l = MD5_Nl;
    unsigned char *p = (unsigned char *)MD5_data;

    p[MD5_num++] = 0x80;
    if (MD5_num >= MD5_CBLOCK-8)
    {   while (MD5_num < MD5_CBLOCK) p[MD5_num++] = 0;
        md5_block();
        MD5_num = 0;
    }
    while (MD5_num < MD5_CBLOCK-8) p[MD5_num++] = 0;
    p[MD5_num++] = l;
    p[MD5_num++] = l>>8;
    p[MD5_num++] = l>>16;
    p[MD5_num++] = l>>24;
    p[MD5_num++] = 0;
    p[MD5_num++] = 0;
    p[MD5_num++] = 0;
    p[MD5_num++] = 0;
    md5_block();
    p = md;
    l = MD5_A; *p++ = l; *p++ = l>>8; *p++ = l>>16; *p++ = l>>24;
    l = MD5_B; *p++ = l; *p++ = l>>8; *p++ = l>>16; *p++ = l>>24;
    l = MD5_C; *p++ = l; *p++ = l>>8; *p++ = l>>16; *p++ = l>>24;
    l = MD5_D; *p++ = l; *p++ = l>>8; *p++ = l>>16; *p++ = l>>24;
}

unsigned char* arcs::MD5( unsigned char *d, int n, unsigned char *md )
{
//  if (n < 0) n = strlen(d);	
    MD5_Init();
    MD5_Update(d, n);
    MD5_Final(md);
    return md;
}

void arcs::MDString( char *in, unsigned char *out, int len )
{
    int i;
    unsigned char mm[16];
    MD5((unsigned char *)in, len, mm);
    for(i=0; i<16; i++) out[i]=mm[i];
}


/* ARCS algorithm */
int arcs::core( char *src, char *dest, char *pwd )
{
	/* Variable Declaration */
	register int x, streamMd5pwd, counter	;
	int i, len, endFile			;
	unsigned char crypted[BLOCK_LENGTH]	;
	unsigned char plainBlock[BLOCK_LENGTH]	;
	char anagram[50]			;
	unsigned char MD5pwd[MD5_LENGTH+1]	;
	unsigned char MD5pwdOne[MD5_LENGTH+1]	;
	unsigned char MD5pwdTwo[MD5_LENGTH+1]	;
	unsigned char MD5temp[MD5_LENGTH*3+1]	;
	FILE *sourceFile, *destinationFile	;
	/* END of Variables Declaration */

	/* Variables Initialization */
	endFile = 0; streamMd5pwd = 0		;
	MD5pwd[MD5_LENGTH] = '\0'		;
	MD5pwdOne[MD5_LENGTH] = '\0'		;
	MD5pwdTwo[MD5_LENGTH] = '\0'		;
	MD5temp[MD5_LENGTH*3] = '\0'		;
	/* END of Variables Initalization */

	strcpy( anagram, pwd )			;			// Copying password in anagram

	/* Starting the Anagram's Generation */
	anagram[0] = pwd[strlen(pwd)-3] 	;
	anagram[1] = pwd[strlen(pwd)-1]		;
	anagram[2] = pwd[strlen(pwd)-2]		;
	anagram[strlen(anagram)-1] = pwd[2] 	;
	anagram[strlen(anagram)-2] = pwd[1]	;
	anagram[strlen(anagram)-3] = pwd[0]	;
	/* END of Anagram's Generation */

	/* Opening Source and Destination Files */
	sourceFile = fopen( src, "r" )			;	
	//assert( sourceFile > 0 )			;
	if( sourceFile < 0 || sourceFile == NULL )
	{ 
		printf( "%s does not exist!\n", src )	;
		return ( -1 )				;
	}
        destinationFile = fopen( dest, "w" )		;
        //assert( destinationFile > 0 )			;
        if( destinationFile < 0 ) return ( -1 )	;

        /* Worm's Generation */
	MDString( pwd, MD5pwd , strlen(pwd) )		;		// Getting Password's MD5 -> MD5pwd
        MDString( anagram, MD5pwdOne , strlen(anagram) );		// Getting Anagram's MD5 -> MD5pwdOne

//	strcpy( MD5pwdTwo, MD5pwd )			;		// Copying MD5pwd in MD5pwdTwo
	for( i=0; i<MD5_LENGTH; i++ )
		MD5pwdTwo[i] = MD5pwd[i]		;
	
	for( i=MD5_LENGTH; i<MD5_LENGTH*2; i++ )
		MD5temp[i] = MD5pwdTwo[i-MD5_LENGTH]	;		// MD5temp[..] = MD5pwd
	for( i=MD5_LENGTH*2; i<MD5_LENGTH*3; i++ )
		MD5temp[i] = MD5pwdOne[i-MD5_LENGTH*2]	;		// MD5temp[..] = MD5pwdOne
	MD5temp[MD5_LENGTH*3] = '\0'			;

	/***********************************************
	*            MD5temp ->  Scheme			*
	*	____16_________32___________48_____	*
	*      |          |           |            |    *
	*      |NULL      |MD5_Pwd    | MD5pwdOne  |    *
	*      |__________|___________|____________|    *
	*************************************************/


        for( counter=0; counter<MD5_LENGTH; counter++ )
    	{
    		/* Shaking */
    		/* MD5temp[x] = MDpwdOne && MD5temp[y] = MDpwdDue , x !=y && x++ && y++ */
          	if( counter % 2 == 0 )
          	    MD5temp[counter] = MD5pwdOne[counter]	;
          	else
          	    MD5temp[counter] = MD5pwdTwo[counter]	;
        }

        /*******************************************************
	*                     MD5temp -> Scheme			*
	*  _____________16__________________32_________48_____	*
	* |     x!=y && x++ && y++ 	|        |            | *
	* |MD5pwdOne[x]    MD5pwd[y] 	|MD5_Pwd | MD5pwdOne  | *
	* |____________________________ |________|____________| *
	*********************************************************/


      	MDString( (char *)MD5temp, MD5pwd , MD5_LENGTH*3 )     ;		// Getting the MD5temp's MD5 in MD5pwd


      	len = BLOCK_LENGTH	;
	do
	{
		x = 0			;
		while ( x < BLOCK_LENGTH )
		{
			plainBlock[x] = fgetc( sourceFile )	;
			if( plainBlock[x] == 255 )
				if( feof( sourceFile ))
				{
					len = x		;
					endFile = 1	;
					break		;
				}
			x = x + 1 	;
		}
		for( x=0; x<len; x++ )
		{
			crypted[x] = plainBlock[x] ^ MD5pwd[streamMd5pwd]	;
			streamMd5pwd++		;
			if( streamMd5pwd == MD5_LENGTH )		// Text Length >= MD5_LENGTH
			{
			        for( counter=0; counter<MD5_LENGTH; counter++ )
    				{
    					/* MD5temp[1/2] = MD5pwd[1/2] */
    					if( counter % 2 == 0 )
		         	    		MD5temp[counter] = MD5pwd[counter]	;
		         	    	/* MD5temp[1/2] = MD5pwd[1/2] */
		         	    	else
		         	    		MD5temp[counter] = MD5pwdOne[counter]	;
        			}
			        for( counter=0; counter<MD5_LENGTH; counter++ )
    				{
		         	    MD5pwdOne[counter] = MD5pwd[counter]	;
        			}
        			
        	       		/*******************************************************************
				 *                     MD5temp -> Scheme			    * 
				 *  ______________16___________________32________________48_______  *
				 * |     x!=y && x++ && y++	|                 |               | *
				 * | MD5pwd[x]    MD5pwdOne[y] 	| Password's Md5  | Anagram's MD5 | *
				 * |____________________________|________________ |_______________| *
				 ********************************************************************/
        			
				MDString( (char *)MD5temp, MD5pwd , MD5_LENGTH*3 )	;
				streamMd5pwd = 0				;
			}
		}
		fwrite( crypted, len, 1, destinationFile )	;

	}
	while( endFile == 0 ) 		;

	fclose( sourceFile )	 	;
	fclose( destinationFile )	;

	return 0			;

}


