/***************************************************************************
 *   Copyright (C) 2005 by Angelo Rosiello   *
 *   angelo@rosiello.org   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef ARCS_H
#define ARCS_H

#include <qstring.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <assert.h>
#include <stdlib.h>
#include <sys/types.h>

#define MD5_LENGTH 16
#define BLOCK_LENGTH 2048
#define MD5_CBLOCK         64
#define MD5_LBLOCK         16

#define F(x,y,z)        ((((y) ^ (z)) & (x)) ^ (z))
#define G(x,y,z)        ((((x) ^ (y)) & (z)) ^ (y))
#define H(x,y,z)        ((x) ^ (y) ^ (z))
#define I(x,y,z)        (((x) | (~(z))) ^ (y))

#define ROTATE(a,n)     (((a)<<(n))|((a)>>(32-(n))))


#define R0(a,b,c,d,k,s,t) {          \
        a+=((k)+(t)+F((b),(c),(d))); \
        a=ROTATE(a,s);               \
        a+=b; }

#define R1(a,b,c,d,k,s,t) {          \
        a+=((k)+(t)+G((b),(c),(d))); \
        a=ROTATE(a,s);               \
        a+=b; }

#define R2(a,b,c,d,k,s,t) {          \
        a+=((k)+(t)+H((b),(c),(d))); \
        a=ROTATE(a,s);               \
        a+=b; }

#define R3(a,b,c,d,k,s,t) {          \
        a+=((k)+(t)+I((b),(c),(d))); \
        a=ROTATE(a,s);               \
        a+=b; }


typedef int int32;
typedef unsigned int unsigned32;

using namespace std;

/**
 * @short ARCS cipher
 * @author Angelo Rosiello <angelo@rosiello.org>
 * @version 0.1
 */
class arcs
{
  public:

    /**
    * Default constructor
    */
    arcs( );

    /**
    * Default destructor
    */
    ~arcs( );

    /**
    * Encrypt function
    * @param source_file Source file to encrypt
    * @param dest_file Destination of the encrypted file
    * @param pwd Password to encrypt
    */
    int encrypt( QString source_file, QString dest_file, QString pwd );

    /**
    * Decrypt function
    * @param source_file Source file to decrypt
    * @param dest_file Destination of the decrypted file
    * @param pwd Password to decrypt
    */
    int decrypt( QString source_file, QString dest_file, QString pwd );

  private:

    unsigned32 MD5_A, MD5_B, MD5_C, MD5_D;
    unsigned32 MD5_Nl;
    int MD5_num;
    unsigned32 MD5_data[MD5_CBLOCK];
    unsigned char byte_order_test[4];

    int core( char *src, char *dest, char *pwd );
    void MDString( char *in, unsigned char *out , int len )	;
    void MD5_Init( )				;
    void md5_block( )			;
    void MD5_Update( unsigned char *data, int len )	;
    void MD5_Final( unsigned char *md )		;
    unsigned char *MD5( unsigned char *d, int n, unsigned char *md );
};

#endif
