/***************************************************************************
 *   Copyright (C) 2005 by Angelo Rosiello   *
 *   angelo@rosiello.org   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "kryptor.h"

//constructor
Kryptor::Kryptor( ): KMainWindow( 0, "Kryptor" )
{
  setXMLFile( "kryptorui.rc" );
  //Popup Menus
  file = new KPopupMenu( );
  file->insertItem( "E&xit", this, SLOT(programExit()), CTRL+Key_Q );

  help = helpMenu( "Kryptor 0.1 by Angelo P. E. Rosiello\nCopyright (C) 2005 Rosiello Security\n http://www.rosiello.org\n\nThis program is free software; you can redistribute it and/or modify it\n under the terms of the GNU General Public License as published\n by the Free Software Foundation" );

  menu = menuBar( );
  menu->insertItem( "&File", file );
  menu->insertItem( "&Help", help );

  //ToolBar
  KIconLoader *loader = new KIconLoader;
  toolbar = new KToolBar( this );
  const int buttonid=1;
  QPixmap encryptico, decryptico;

  if( QFileInfo( "/usr/share/pixmaps/encrypt.png" ).exists()==TRUE )
    encryptico=QPixmap( "/usr/share/pixmaps/encrypt.png" );
  else if( QFileInfo( "/usr/local/share/pixmaps/encrypt.png" ).exists()==TRUE )
    encryptico=QPixmap( "/usr/local/share/pixmaps/encrypt.png" );
  else
    encryptico=QPixmap( "/usr/local/kde/share/pixmaps/padlock.png" );

  if( QFileInfo( "/usr/share/pixmaps/decrypt.png" ).exists()==TRUE )
    decryptico=QPixmap( "/usr/share/pixmaps/decrypt.png" );
  else if( QFileInfo( "/usr/local/share/pixmaps/decrypt.png" ).exists()==TRUE )
    decryptico=QPixmap( "/usr/local/share/pixmaps/decrypt.png" );
  else 
    decryptico=QPixmap( "/usr/local/kde/share/pixmaps/decrypt.png" );

  //insert encrypt button
  toolbar->insertButton( encryptico,
                         buttonid,SIGNAL(clicked()),this,SLOT(encrypt()),TRUE,
                          "Encrypt" );

  //insert decrypt button
  toolbar->insertButton( decryptico,
                         buttonid,SIGNAL(clicked()),this,SLOT(decrypt()),TRUE,
                          "Decrypt" );

  //insert file erase button
  //insert exit button
  toolbar->insertButton(loader->loadIcon((QString)"edittrash.xpm", KIcon::NoGroup),
                         buttonid,SIGNAL(clicked()),this,SLOT(fileErase()),TRUE,
                          "Secure Erase");

  //insert exit button
  toolbar->insertButton(loader->loadIcon((QString)"exit.xpm", KIcon::NoGroup),
                         buttonid,SIGNAL(clicked()),this,SLOT(programExit()),TRUE,
                          "Quit");
  addToolBar(toolbar);
  toolbar->setBarPos(KToolBar::Top);

  //KActionSelector
  view = new KActionSelector( this, "Panel" );
  initializeSelector( );
  setCentralWidget( view );

  //status Bar
  statusBar = new KStatusBar(this); 
  statusBar->message("Kryptor is Ready!");
}

Kryptor::~Kryptor()
{
}

void Kryptor::initializeSelector( )
{
  //!(doubleclick -> insertItem)
  view->setMoveOnDoubleClick( FALSE );
  available=view->availableListBox( );
  selected=view->selectedListBox( );
  //fill selected with current files
  setDir( QDir::currentDirPath() );
  connect( available, SIGNAL(selected(int)), this, SLOT(availableItemSelected(int)) );
}

void Kryptor::setDir( const QString &path )
{
  QDir dir( path, "*", QDir::DirsFirst );
  dir.setMatchAllDirs( TRUE );
  if( !dir.isReadable( ) )
    return;
  available->clear( );
  basePath = dir.canonicalPath( );
  QStringList entries = dir.entryList( );
  QStringList::ConstIterator it = entries.constBegin( );
  while( it!=entries.constEnd( ) ) 
  {
    if( *it!=( "." ) )
    {
      if( *it!=".." )
      {
         if( basePath.compare("/")!=0 )
           available->insertItem( basePath+"/"+*it );
         else
           available->insertItem( "/"+*it );
      }
      else
      {
         available->insertItem( *it );
      }
    }
    ++it;
  }
  available->sort( );
}

void Kryptor::availableItemSelected( int index )
{
  QString path, temp;
  int pos;
  bool cs;
  path = basePath;
  if( available->text(index)==".." )
  {
    pos=path.findRev( '/', -1, cs=TRUE );
    if( pos!=0 )
    { 
      temp=path.mid( 0, pos );
      path=temp;
    }
    else
    {
      path="/";
    }
  }
  else
    path = available->text( index );

  if ( QFileInfo(path).isDir() )
       setDir( path );
}

//encrypt routine
void Kryptor::encrypt( )
{
  QString efilename, sctr, filextension=".arcs";
  int nfiles=0,ctr;
  char data[128];
  arcs arcEncrypt;

  //if list is empty alert
  if( selected->count()==0 )
  {
    KMessageBox::information( this, "No File Selected!" );
    return;
  }
  //else ask for password and encrypt
  //QString pwd = KLineEditDlg::getText( i18n("Insert Password"),i18n("Password:") );
  QCString pwd;
  int result = KPasswordDialog::getPassword(pwd, i18n("Password"));
  if (result != KPasswordDialog::Accepted)
  {
    return;
  }   
  // 6<pwd<40
  if( pwd.length()<6 || pwd.length()>30 )
  {
    KMessageBox::error( 0, "Password invalid, too long or too short!\nPush encrypt again!", "Error" );
    return;
  }
   
  //check selected ListBox to get all files
  QProgressDialog *progress=new QProgressDialog( "Encryption in progress...", "Abort", (int)selected->count(), this, "progress", TRUE );
  progress->setCaption( "Please Wait" );
  progress->show( );

  for( int i=0; i<(int)selected->count(); i++ )
  {
    progress->setProgress( i );
    progress->show();
    qApp->processEvents( );
    if ( progress->wasCancelled() )
    {
      setDir( QDir::currentDirPath() );
      selected->clear( );
      delete progress;
      return;
    }
    if( QFileInfo(selected->text(i)).isFile()==TRUE )
    {
      efilename=selected->text(i)+filextension;
      if( QFileInfo(efilename).isFile()==TRUE )
      {
        if( KMessageBox::Cancel==KMessageBox::warningContinueCancel(this, i18n("The file " + 
              (QString)selected->text(i)+filextension+" already exists!\nDo you wanto to overwrite it?")) ) 
        { 
          goto cycle;
        }
      }
    }
    else if( QFileInfo(selected->text(i)).isDir()==TRUE )
    {
      if( KMessageBox::Cancel==KMessageBox::warningContinueCancel(this, i18n( (QString)selected->text(i)+" is a directory and can't be encrypted!\nDo you wanto to continue encrypting the other selected files?")) ) 
      { 
        delete progress;
        return;
      }
      else 
        goto cycle;
    }
    ctr=(int)QFileInfo( selected->text(i) ).size( );
    sprintf( data, "%i", ctr );
    sctr=QString( data );
    //padding the pwd for more security because ARCS is a stream cipher!
    pwd=pwd+data;
    if( arcEncrypt.encrypt( (QString)selected->text( i ), (QString)selected->text( i )+filextension, pwd )==-1 )
    {
      KMessageBox::error( 0, (QString)"Bad encryption!\nFile:"+selected->text(i), "Error" );
      delete progress;
      return;
    }

    if( nfiles>65535 )
      return;
    nfiles++;
    cycle:
      continue;
  }
  progress->setProgress( (int)selected->count() );
  selected->clear( );
  sprintf( data, "%i", nfiles );
  QString snfiles( data );
  KMessageBox::information( this, snfiles+" file(s) were correctly encrypted!" );

  setDir( QDir::currentDirPath() );
  delete progress;
}

//decrypt routine
void Kryptor::decrypt( )
{
  QString dfilename, sctr, filextension=".arcs"; 
  int nfiles=0,pos, ctr; bool cs;
  char data[128];
  arcs arcDecrypt; 

  //if list is empty alert
  if( selected->count()==0 )
  {
    KMessageBox::information( this, "No File Selected!" );
    return;
  }

  //else ask for password and encrypt
  QCString pwd;
  int result = KPasswordDialog::getPassword(pwd, i18n("Password"));
  if (result != KPasswordDialog::Accepted)
  {
    return;
  }   
  // 6<pwd<40
  if( pwd.length()<6 || pwd.length()>30 )
  {
    KMessageBox::error( 0, "Password invalid, too long or too short!\nPush decrypt again!", "Error" );
    return;
  }

  QProgressDialog *progress=new QProgressDialog( "Decryption in progress...", "Abort", (int)selected->count(), this, "progress", TRUE );
  progress->setCaption( "Please Wait" );
  progress->show();

  for( int i=0; i<(int)selected->count(); i++ )
  {
    progress->setProgress( i );
    progress->show( );
    qApp->processEvents( );
    if ( progress->wasCancelled() )
    {
      setDir( QDir::currentDirPath() );
      selected->clear( );
      delete progress;
      return;
    }
    dfilename=selected->text( i );
    if( QFileInfo(selected->text(i)).isFile()==TRUE )
    {
      pos=selected->text(i).findRev( '.', -1, cs=TRUE );
      if( pos!=-1 ) 
      {
        QString extension=selected->text(i).mid( pos, selected->text(i).length() );
        if( extension.compare( ".arcs" )==0 )
        {
          dfilename=selected->text(i).mid( 0, pos )+".clr";
        }
        else
        {
            dfilename=selected->text(i)+".clr";
        }
      }
      else
      {
        dfilename=selected->text(i)+".clr";
      }
      if( QFileInfo(dfilename).isFile()==TRUE )
      {
        if( KMessageBox::Cancel==KMessageBox::warningContinueCancel(this, i18n("The file " + 
              dfilename+" already exists!\nDo you wanto to overwrite it?")) )
        {
          goto cycle;
        }
      }
    }
    else if( QFileInfo(selected->text(i)).isDir()==TRUE )
    {
      if( KMessageBox::Cancel==KMessageBox::warningContinueCancel(this, i18n( (QString)selected->text(i)+" is a directory and can't be decrypted!\nDo you wanto to continue decrypting the other selected files?")) ) 
      { 
        delete progress;
        return;
      }
      else 
        goto cycle;
    }
    ctr=(int)QFileInfo( selected->text(i) ).size();
    sprintf( data, "%i", ctr );
    sctr=QString( data );
    pwd=pwd+data;
    if( arcDecrypt.decrypt( selected->text(i), dfilename, pwd )==-1 )
    {
      KMessageBox::error( 0, (QString)"Bad decryption!\nFile:"+selected->text(i), "Error" );
      delete progress;
      return;
    }
    if( nfiles>65535 )
      return;
    nfiles++;
    cycle:
      continue;
  }
  progress->setProgress( (int)selected->count() );
  selected->clear( );
  sprintf( data, "%i", nfiles );
  QString snfiles( data );
  KMessageBox::information( this, snfiles+" file(s) were correctly decrypted!" );

  setDir( QDir::currentDirPath() );

  delete progress;
}

//file erase
void Kryptor::fileErase( )
{
  int nfiles=0, seq[256]; 
  char data[128];
  unsigned int seed; 
  bool end; 
  //if list is empty alert
  if( selected->count()==0 )
  {
    KMessageBox::information( this, "No File Selected!" );
    return;
  }
  QProgressDialog *progress=new QProgressDialog( "Erasing in progress...", "Abort", (int)selected->count(), this, "progress", TRUE );
  progress->setCaption( "Please Wait" );
  progress->show();

  for( int i=0; i<(int)selected->count(); i++ )
  {
    progress->setProgress( i );
    progress->show( );
    qApp->processEvents( );
    if ( progress->wasCancelled() )
    {
      setDir( QDir::currentDirPath() );
      selected->clear( );
      delete progress;
      return;
    }
    if( QFileInfo(selected->text(i)).isFile()==TRUE )
    {
      //get size of file
      struct stat st;
      if( stat((const char *)selected->text(i).latin1(), &st) != 0 )
        return;
      int size = st.st_size;  // size in bytes
      //std::cout << "SIZE: " << size << std::endl;

      FILE *myFile=fopen( selected->text(i), "r+" );
      if( myFile==NULL )
      {
        KMessageBox::error( 0, (QString)"Error while erasing file:"+selected->text(i), "Error" );
        delete progress;
        return;
      }


      //Overwriting the file 3 times with random data
      for( int j=0; j<3; j++ )
      {
        end=FALSE;
        rewind( myFile ); 
        ifstream randfile( "/dev/random" );
        if ( !randfile )
        {
          KMessageBox::error( 0, (QString)"This system doesn't have /dev/random", "Error" );
          delete progress;
          return;
        }
        seed=randfile.get( );
        randfile.close( ); 
        srand( seed );
        while( end==FALSE )
        {
          for( int x=0; x<256; x++ )
          {
            seq[x]=rand();
          }
          fwrite( &seq, sizeof(int), 256 , myFile );
          if( ftell( myFile )>=size )
          {
            end=TRUE;
          }
        }
	fflush( myFile ); 
      }
      fclose( myFile );
      remove( selected->text(i) );
    }
    else if( QFileInfo(selected->text(i)).isDir()==TRUE )
    {
      if( KMessageBox::Cancel==KMessageBox::warningContinueCancel(this, i18n( (QString)selected->text(i)+" is a directory and can't be secure erased\nDo you wanto to continue erasing the other selected files?")) ) 
      { 
        delete progress;
        return;
      }
      else 
        goto cycle;
    }
    if( nfiles>65535 )
      return;
    nfiles++;
    cycle:
      continue;
  }
  progress->setProgress( (int)selected->count() );
  selected->clear( );
  sprintf( data, "%i", nfiles );
  QString snfiles( data );
  KMessageBox::information( this, snfiles+" file(s) were correctly erased!" );

  setDir( QDir::currentDirPath() );

  delete progress;

}

//exit( )
void Kryptor::programExit( )
{
  statusBar->message( "Exiting..." );
  close( );
}


#include "kryptor.moc"
